@@ -1,22 +1,14 @@
 'use strict';
 
-// Load modules
+const Code = require('@hapi/code');
+const Hawk = require('..');
+const Lab = require('@hapi/lab');
 
-const Code = require('code');
-const Hawk = require('../lib');
-const Lab = require('lab');
-
-
-// Declare internals
 
 const internals = {};
 
 
-// Test shortcuts
-
-const lab = exports.lab = Lab.script();
-const describe = lab.experiment;
-const it = lab.test;
+const { describe, it } = exports.lab = Lab.script();
 const expect = Code.expect;
 
 
@@ -24,7 +16,7 @@ describe('Client', () => {
 
     describe('header()', () => {
 
-        it('returns a valid authorization header (sha1)', (done) => {
+        it('returns a valid authorization header (sha1)', () => {
 
             const credentials = {
                 id: '123456',
@@ -32,12 +24,11 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const header = Hawk.client.header('http://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about' }).field;
+            const { header } = Hawk.client.header('http://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about' });
             expect(header).to.equal('Hawk id="123456", ts="1353809207", nonce="Ygvqdz", hash="bsvY3IfUllw6V5rvk4tStEvpBhE=", ext="Bazinga!", mac="qbf1ZPG/r/e06F4ht+T77LXi5vw="');
-            done();
         });
 
-        it('returns a valid authorization header (sha256)', (done) => {
+        it('returns a valid authorization header (sha256)', () => {
 
             const credentials = {
                 id: '123456',
@@ -45,12 +36,11 @@ describe('Client', () => {
                 algorithm: 'sha256'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain' }).field;
+            const { header } = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain' });
             expect(header).to.equal('Hawk id="123456", ts="1353809207", nonce="Ygvqdz", hash="2QfCt3GuY9HQnHWyWD3wX68ZOKbynqlfYmuO2ZBRqtY=", ext="Bazinga!", mac="q1CwFoSHzPZSkbIvl0oYlD+91rBUEvFk763nMjMndj8="');
-            done();
         });
 
-        it('returns a valid authorization header (no ext)', (done) => {
+        it('returns a valid authorization header (no ext)', () => {
 
             const credentials = {
                 id: '123456',
@@ -58,12 +48,11 @@ describe('Client', () => {
                 algorithm: 'sha256'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain' }).field;
+            const { header } = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain' });
             expect(header).to.equal('Hawk id="123456", ts="1353809207", nonce="Ygvqdz", hash="2QfCt3GuY9HQnHWyWD3wX68ZOKbynqlfYmuO2ZBRqtY=", mac="HTgtd0jPI6E4izx8e4OHdO36q00xFCU0FolNq3RiCYs="');
-            done();
         });
 
-        it('returns a valid authorization header (null ext)', (done) => {
+        it('returns a valid authorization header (null ext)', () => {
 
             const credentials = {
                 id: '123456',
@@ -71,12 +60,11 @@ describe('Client', () => {
                 algorithm: 'sha256'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain', ext: null }).field;
+            const { header } = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain', ext: null });
             expect(header).to.equal('Hawk id="123456", ts="1353809207", nonce="Ygvqdz", hash="2QfCt3GuY9HQnHWyWD3wX68ZOKbynqlfYmuO2ZBRqtY=", mac="HTgtd0jPI6E4izx8e4OHdO36q00xFCU0FolNq3RiCYs="');
-            done();
         });
 
-        it('returns a valid authorization header (empty payload)', (done) => {
+        it('returns a valid authorization header (empty payload)', () => {
 
             const credentials = {
                 id: '123456',
@@ -84,12 +72,11 @@ describe('Client', () => {
                 algorithm: 'sha256'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: '', contentType: 'text/plain' }).field;
+            const { header } = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: '', contentType: 'text/plain' });
             expect(header).to.equal('Hawk id=\"123456\", ts=\"1353809207\", nonce=\"Ygvqdz\", hash=\"q/t+NNAkQZNlq/aAD6PlexImwQTxwgT2MahfTa9XRLA=\", mac=\"U5k16YEzn3UnBHKeBzsDXn067Gu3R4YaY6xOt9PYRZM=\"');
-            done();
         });
 
-        it('returns a valid authorization header (pre hashed payload)', (done) => {
+        it('returns a valid authorization header (pre hashed payload)', () => {
 
             const credentials = {
                 id: '123456',
@@ -99,86 +86,76 @@ describe('Client', () => {
 
             const options = { credentials, timestamp: 1353809207, nonce: 'Ygvqdz', payload: 'something to write about', contentType: 'text/plain' };
             options.hash = Hawk.crypto.calculatePayloadHash(options.payload, credentials.algorithm, options.contentType);
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', options).field;
+            const { header } = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', options);
             expect(header).to.equal('Hawk id="123456", ts="1353809207", nonce="Ygvqdz", hash="2QfCt3GuY9HQnHWyWD3wX68ZOKbynqlfYmuO2ZBRqtY=", mac="HTgtd0jPI6E4izx8e4OHdO36q00xFCU0FolNq3RiCYs="');
-            done();
         });
 
-        it('errors on missing uri', (done) => {
+        it('errors on missing uri', () => {
 
-            const header = Hawk.client.header('', 'POST');
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid argument type');
-            done();
+            expect(() => Hawk.client.header('', 'POST')).to.throw('Invalid argument type');
         });
 
-        it('errors on invalid uri', (done) => {
+        it('errors on invalid uri', () => {
 
-            const header = Hawk.client.header(4, 'POST');
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid argument type');
-            done();
+            expect(() => Hawk.client.header(4, 'POST')).to.throw('Invalid argument type');
         });
 
-        it('errors on missing method', (done) => {
+        it('errors on missing method', () => {
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', '');
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid argument type');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', '')).to.throw('Invalid argument type');
         });
 
-        it('errors on invalid method', (done) => {
+        it('errors on invalid method', () => {
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 5);
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid argument type');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 5)).to.throw('Invalid argument type');
         });
 
-        it('errors on missing options', (done) => {
+        it('errors on missing options', () => {
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST');
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid argument type');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST')).to.throw('Invalid argument type');
         });
 
-        it('errors on invalid credentials (id)', (done) => {
+        it('errors on invalid options', () => {
+
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', 'abc')).to.throw('Invalid argument type');
+        });
+
+        it('errors on invalid credentials (id)', () => {
 
             const credentials = {
                 key: '2983d45yun89q',
                 algorithm: 'sha256'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207 });
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid credential object');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207 })).to.throw('Invalid credentials');
         });
 
-        it('errors on missing credentials', (done) => {
+        it('errors on missing credentials', () => {
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { ext: 'Bazinga!', timestamp: 1353809207 });
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid credential object');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { ext: 'Bazinga!', timestamp: 1353809207 })).to.throw('Invalid credentials');
         });
 
-        it('errors on invalid credentials', (done) => {
+        it('errors on invalid credentials (key)', () => {
 
             const credentials = {
                 id: '123456',
                 algorithm: 'sha256'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207 });
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Invalid credential object');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207 })).to.throw('Invalid credentials');
         });
 
-        it('errors on invalid algorithm', (done) => {
+        it('errors on invalid credentials (algorithm)', () => {
+
+            const credentials = {
+                id: '123456',
+                key: 'asdasdasd'
+            };
+
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, ext: 'Bazinga!', timestamp: 1353809207 })).to.throw('Invalid credentials');
+        });
+
+        it('errors on invalid algorithm', () => {
 
             const credentials = {
                 id: '123456',
@@ -186,16 +163,13 @@ describe('Client', () => {
                 algorithm: 'hmac-sha-0'
             };
 
-            const header = Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, payload: 'something, anything!', ext: 'Bazinga!', timestamp: 1353809207 });
-            expect(header.field).to.equal('');
-            expect(header.err).to.equal('Unknown algorithm');
-            done();
+            expect(() => Hawk.client.header('https://example.net/somewhere/over/the/rainbow', 'POST', { credentials, payload: 'something, anything!', ext: 'Bazinga!', timestamp: 1353809207 })).to.throw('Unknown algorithm');
         });
     });
 
     describe('authenticate()', () => {
 
-        it('returns false on invalid header', (done) => {
+        it('rejects on invalid header', () => {
 
             const res = {
                 headers: {
@@ -203,27 +177,10 @@ describe('Client', () => {
                 }
             };
 
-            expect(Hawk.client.authenticate(res, {})).to.equal(false);
-            done();
+            expect(() => Hawk.client.authenticate(res)).to.throw('Invalid Server-Authorization header');
         });
 
-        it('returns false on invalid header (callback)', (done) => {
-
-            const res = {
-                headers: {
-                    'server-authorization': 'Hawk mac="abc", bad="xyz"'
-                }
-            };
-
-            Hawk.client.authenticate(res, {}, null, null, (err) => {
-
-                expect(err).to.exist();
-                expect(err.message).to.equal('Invalid Server-Authorization header');
-                done();
-            });
-        });
-
-        it('returns false on invalid mac', (done) => {
+        it('rejects on invalid mac', () => {
 
             const res = {
                 headers: {
@@ -254,11 +211,10 @@ describe('Client', () => {
                 user: 'steve'
             };
 
-            expect(Hawk.client.authenticate(res, credentials, artifacts)).to.equal(false);
-            done();
+            expect(() => Hawk.client.authenticate(res, credentials, artifacts)).to.throw('Bad response mac');
         });
 
-        it('returns true on ignoring hash', (done) => {
+        it('returns headers on ignoring hash', () => {
 
             const res = {
                 headers: {
@@ -289,96 +245,60 @@ describe('Client', () => {
                 user: 'steve'
             };
 
-            expect(Hawk.client.authenticate(res, credentials, artifacts)).to.equal(true);
-            done();
-        });
-
-        it('validates response payload', (done) => {
-
-            const payload = 'some reply';
-
-            const res = {
-                headers: {
-                    'content-type': 'text/plain',
-                    'server-authorization': 'Hawk mac="odsVGUq0rCoITaiNagW22REIpqkwP9zt5FyqqOW9Zj8=", hash="f9cDF/TDm7TkYRLnGwRMfeDzT6LixQVLvrIKhh0vgmM=", ext="response-specific"'
+            const { headers } = Hawk.client.authenticate(res, credentials, artifacts, { payload: null });
+            expect(headers).to.equal({
+                'server-authorization': {
+                    mac: 'XIJRsMl/4oL+nn+vKoeVZPdCHXB4yJkNnBbTbHFZUYE=',
+                    hash: 'f9cDF/TDm7TkYRLnGwRMfeDzT6LixQVLvrIKhh0vgmM=',
+                    ext: 'response-specific'
                 }
-            };
-
-            const credentials = {
-                id: '123456',
-                key: 'werxhqb98rpaxn39848xrunpaw3489ruxnpa98w4rxn',
-                algorithm: 'sha256',
-                user: 'steve'
-            };
-
-            const artifacts = {
-                method: 'POST',
-                host: 'example.com',
-                port: '8080',
-                resource: '/resource/4?filter=a',
-                ts: '1453070933',
-                nonce: '3hOHpR',
-                hash: 'nJjkVtBE5Y/Bk38Aiokwn0jiJxt/0S2WRSUwWLCf5xk=',
-                ext: 'some-app-data',
-                app: undefined,
-                dlg: undefined,
-                mac: '/DitzeD66F2f7O535SERbX9p+oh9ZnNLqSNHG+c7/vs=',
-                id: '123456'
-            };
-
-            expect(Hawk.client.authenticate(res, credentials, artifacts, { payload })).to.equal(true);
-            done();
-        });
-
-        it('validates response payload (callback)', (done) => {
-
-            const payload = 'some reply';
-
-            const res = {
-                headers: {
-                    'content-type': 'text/plain',
-                    'server-authorization': 'Hawk mac="odsVGUq0rCoITaiNagW22REIpqkwP9zt5FyqqOW9Zj8=", hash="f9cDF/TDm7TkYRLnGwRMfeDzT6LixQVLvrIKhh0vgmM=", ext="response-specific"'
-                }
-            };
-
-            const credentials = {
-                id: '123456',
-                key: 'werxhqb98rpaxn39848xrunpaw3489ruxnpa98w4rxn',
-                algorithm: 'sha256',
-                user: 'steve'
-            };
-
-            const artifacts = {
-                method: 'POST',
-                host: 'example.com',
-                port: '8080',
-                resource: '/resource/4?filter=a',
-                ts: '1453070933',
-                nonce: '3hOHpR',
-                hash: 'nJjkVtBE5Y/Bk38Aiokwn0jiJxt/0S2WRSUwWLCf5xk=',
-                ext: 'some-app-data',
-                app: undefined,
-                dlg: undefined,
-                mac: '/DitzeD66F2f7O535SERbX9p+oh9ZnNLqSNHG+c7/vs=',
-                id: '123456'
-            };
-
-            Hawk.client.authenticate(res, credentials, artifacts, { payload }, (err, headers) => {
-
-                expect(err).to.not.exist();
-                expect(headers).to.equal({
-                    'www-authenticate': null,
-                    'server-authorization': {
-                        mac: 'odsVGUq0rCoITaiNagW22REIpqkwP9zt5FyqqOW9Zj8=',
-                        hash: 'f9cDF/TDm7TkYRLnGwRMfeDzT6LixQVLvrIKhh0vgmM=',
-                        ext: 'response-specific'
-                    }
-                });
-                done();
             });
         });
 
-        it('errors on invalid response payload', (done) => {
+        it('validates response payload', () => {
+
+            const payload = 'some reply';
+
+            const res = {
+                headers: {
+                    'content-type': 'text/plain',
+                    'server-authorization': 'Hawk mac="odsVGUq0rCoITaiNagW22REIpqkwP9zt5FyqqOW9Zj8=", hash="f9cDF/TDm7TkYRLnGwRMfeDzT6LixQVLvrIKhh0vgmM=", ext="response-specific"'
+                }
+            };
+
+            const credentials = {
+                id: '123456',
+                key: 'werxhqb98rpaxn39848xrunpaw3489ruxnpa98w4rxn',
+                algorithm: 'sha256',
+                user: 'steve'
+            };
+
+            const artifacts = {
+                method: 'POST',
+                host: 'example.com',
+                port: '8080',
+                resource: '/resource/4?filter=a',
+                ts: '1453070933',
+                nonce: '3hOHpR',
+                hash: 'nJjkVtBE5Y/Bk38Aiokwn0jiJxt/0S2WRSUwWLCf5xk=',
+                ext: 'some-app-data',
+                app: undefined,
+                dlg: undefined,
+                mac: '/DitzeD66F2f7O535SERbX9p+oh9ZnNLqSNHG+c7/vs=',
+                id: '123456'
+            };
+
+            const { headers } = Hawk.client.authenticate(res, credentials, artifacts, { payload });
+            expect(headers).to.equal({
+                'server-authorization': {
+                    mac: 'odsVGUq0rCoITaiNagW22REIpqkwP9zt5FyqqOW9Zj8=',
+                    hash: 'f9cDF/TDm7TkYRLnGwRMfeDzT6LixQVLvrIKhh0vgmM=',
+                    ext: 'response-specific'
+                }
+            });
+        });
+
+        it('errors on invalid response payload', () => {
 
             const payload = 'wrong reply';
 
@@ -411,18 +331,16 @@ describe('Client', () => {
                 id: '123456'
             };
 
-            expect(Hawk.client.authenticate(res, credentials, artifacts, { payload })).to.equal(false);
-            done();
+            expect(() => Hawk.client.authenticate(res, credentials, artifacts, { payload })).to.throw('Bad response payload mac');
         });
 
-        it('fails on invalid WWW-Authenticate header format', (done) => {
+        it('fails on invalid WWW-Authenticate header format', () => {
 
             const header = 'Hawk ts="1362346425875", tsm="PhwayS28vtnn3qbv0mqRBYSXebN/zggEtucfeZ620Zo=", x="Stale timestamp"';
-            expect(Hawk.client.authenticate({ headers: { 'www-authenticate': header } }, {})).to.equal(false);
-            done();
+            expect(() => Hawk.client.authenticate({ headers: { 'www-authenticate': header } }, {})).to.throw('Invalid WWW-Authenticate header');
         });
 
-        it('fails on invalid WWW-Authenticate header format', (done) => {
+        it('fails on invalid WWW-Authenticate header format', () => {
 
             const credentials = {
                 id: '123456',
@@ -432,21 +350,53 @@ describe('Client', () => {
             };
 
             const header = 'Hawk ts="1362346425875", tsm="hwayS28vtnn3qbv0mqRBYSXebN/zggEtucfeZ620Zo=", error="Stale timestamp"';
-            expect(Hawk.client.authenticate({ headers: { 'www-authenticate': header } }, credentials)).to.equal(false);
-            done();
+            expect(() => Hawk.client.authenticate({ headers: { 'www-authenticate': header } }, credentials)).to.throw('Invalid server timestamp hash');
         });
 
-        it('skips tsm validation when missing ts', (done) => {
+        it('skips tsm validation when missing ts', () => {
 
             const header = 'Hawk error="Stale timestamp"';
-            expect(Hawk.client.authenticate({ headers: { 'www-authenticate': header } }, {})).to.equal(true);
-            done();
+            const { headers } = Hawk.client.authenticate({ headers: { 'www-authenticate': header } });
+            expect(headers).to.equal({ 'www-authenticate': { error: 'Stale timestamp' } });
+        });
+
+        it('errors on missing server-authorization header', () => {
+
+            const res = {
+                headers: {
+                    'content-type': 'text/plain'
+                }
+            };
+
+            const artifacts = {
+                method: 'POST',
+                host: 'example.com',
+                port: '8080',
+                resource: '/resource/4?filter=a',
+                ts: '1362336900',
+                nonce: 'eb5S_L',
+                hash: 'nJjkVtBE5Y/Bk38Aiokwn0jiJxt/0S2WRSUwWLCf5xk=',
+                ext: 'some-app-data',
+                app: undefined,
+                dlg: undefined,
+                mac: 'BlmSe8K+pbKIb6YsZCnt4E1GrYvY1AaYayNR82dGpIk=',
+                id: '123456'
+            };
+
+            const credentials = {
+                id: '123456',
+                key: 'werxhqb98rpaxn39848xrunpaw3489ruxnpa98w4rxn',
+                algorithm: 'sha256',
+                user: 'steve'
+            };
+
+            expect(() => Hawk.client.authenticate(res, credentials, artifacts, { required: true })).to.throw('Missing Server-Authorization header');
         });
     });
 
     describe('message()', () => {
 
-        it('generates authorization', (done) => {
+        it('generates authorization', () => {
 
             const credentials = {
                 id: '123456',
@@ -458,10 +408,9 @@ describe('Client', () => {
             expect(auth).to.exist();
             expect(auth.ts).to.equal(1353809207);
             expect(auth.nonce).to.equal('abc123');
-            done();
         });
 
-        it('errors on invalid host', (done) => {
+        it('errors on invalid host', () => {
 
             const credentials = {
                 id: '123456',
@@ -469,12 +418,10 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message(5, 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message(5, 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on invalid port', (done) => {
+        it('errors on invalid port', () => {
 
             const credentials = {
                 id: '123456',
@@ -482,12 +429,10 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', '80', 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', '80', 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on missing host', (done) => {
+        it('errors on missing host', () => {
 
             const credentials = {
                 id: '123456',
@@ -495,12 +440,10 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message(undefined, 0, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message(undefined, 0, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on missing port', (done) => {
+        it('errors on missing port', () => {
 
             const credentials = {
                 id: '123456',
@@ -508,12 +451,10 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', undefined, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', undefined, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on null message', (done) => {
+        it('errors on null message', () => {
 
             const credentials = {
                 id: '123456',
@@ -521,12 +462,10 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', 80, null, { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', 80, null, { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on missing message', (done) => {
+        it('errors on missing message', () => {
 
             const credentials = {
                 id: '123456',
@@ -534,12 +473,10 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', 80, undefined, { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', 80, undefined, { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on invalid message', (done) => {
+        it('errors on invalid message', () => {
 
             const credentials = {
                 id: '123456',
@@ -547,40 +484,47 @@ describe('Client', () => {
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', 80, 5, { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', 80, 5, { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid inputs');
         });
 
-        it('errors on missing options', (done) => {
+        it('errors on invalid credentials', () => {
 
-            const auth = Hawk.client.message('example.com', 80, 'I am the boodyman');
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', 80, 'I am the boodyman')).to.throw('Invalid credentials');
         });
 
-        it('errors on invalid credentials (id)', (done) => {
+        it('errors on invalid options', () => {
+
+            expect(() => Hawk.client.message('example.com', 80, 'I am the boodyman', '123')).to.throw('Invalid inputs');
+        });
+
+        it('errors on invalid credentials (id)', () => {
 
             const credentials = {
                 key: '2983d45yun89q',
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid credentials');
         });
 
-        it('errors on invalid credentials (key)', (done) => {
+        it('errors on invalid credentials (algorithm)', () => {
+
+            const credentials = {
+                key: '2983d45yun89q',
+                id: '123'
+            };
+
+            expect(() => Hawk.client.message('example.com', 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid credentials');
+        });
+
+        it('errors on invalid credentials (key)', () => {
 
             const credentials = {
                 id: '123456',
                 algorithm: 'sha1'
             };
 
-            const auth = Hawk.client.message('example.com', 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' });
-            expect(auth).to.not.exist();
-            done();
+            expect(() => Hawk.client.message('example.com', 80, 'I am the boodyman', { credentials, timestamp: 1353809207, nonce: 'abc123' })).to.throw('Invalid credentials');
         });
     });
 });
